@@ -1,11 +1,12 @@
 """AMQP Connections."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
-from __future__ import absolute_import, unicode_literals
 
 import logging
 import socket
 import uuid
 import warnings
+from array import array
+from time import monotonic
 
 from vine import ensure_promise
 
@@ -16,7 +17,6 @@ from .exceptions import (AMQPDeprecationWarning, ChannelError, ConnectionError,
                          ConnectionForced, RecoverableChannelError,
                          RecoverableConnectionError, ResourceError,
                          error_for_code)
-from .five import array, items, monotonic, range, string, values
 from .method_framing import frame_handler, frame_writer
 from .transport import Transport
 
@@ -39,7 +39,7 @@ START_DEBUG_FMT = """
 Start from server, version: %d.%d, properties: %s, mechanisms: %s, locales: %s
 """.strip()
 
-__all__ = ['Connection']
+__all__ = ('Connection',)
 
 AMQP_LOGGER = logging.getLogger('amqp')
 AMQP_HEARTBEAT_LOGGER = logging.getLogger(
@@ -94,8 +94,8 @@ class Connection(AbstractChannel):
     client name. For EXTERNAL authentication both userid and password are
     ignored.
 
-    The 'ssl' parameter may be simply True/False, or for Python >= 2.6
-    a dictionary of options to pass to ssl.wrap_socket() such as
+    The 'ssl' parameter may be simply True/False, or for Python >= 3.6
+    a dictionary of options to pass to ssl.SSLContext such as
     requiring certain certificates.
 
     The "socket_settings" parameter is a dictionary defining tcp
@@ -244,7 +244,7 @@ class Connection(AbstractChannel):
 
         self.channels = {}
         # The connection object itself is treated as channel 0
-        super(Connection, self).__init__(self, 0)
+        super().__init__(self, 0)
 
         self._frame_writer = None
         self._on_inbound_frame = None
@@ -277,6 +277,14 @@ class Connection(AbstractChannel):
 
         self.connect_timeout = connect_timeout
 
+    def __repr__(self):
+        if self._transport:
+            return f'<AMQP Connection: {self.host}/{self.virtual_host} '\
+                   f'using {self._transport} at {id(self):#x}>'
+        else:
+            return f'<AMQP Connection: {self.host}/{self.virtual_host} '\
+                   f'(disconnected) at {id(self):#x}>'
+
     def __enter__(self):
         self.connect()
         return self
@@ -319,7 +327,7 @@ class Connection(AbstractChannel):
             while not self._handshake_complete:
                 self.drain_events(timeout=self.connect_timeout)
 
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             self.collect()
             raise
 
@@ -366,7 +374,7 @@ class Connection(AbstractChannel):
         self.version_major = version_major
         self.version_minor = version_minor
         self.server_properties = server_properties
-        if isinstance(mechanisms, string):
+        if isinstance(mechanisms, str):
             mechanisms = mechanisms.encode('utf-8')
         self.mechanisms = mechanisms.split(b' ')
         self.locales = locales.split(' ')
@@ -381,7 +389,7 @@ class Connection(AbstractChannel):
         cap = client_properties.setdefault('capabilities', {})
         cap.update({
             wanted_cap: enable_cap
-            for wanted_cap, enable_cap in items(self.negotiate_capabilities)
+            for wanted_cap, enable_cap in self.negotiate_capabilities.items()
             if scap.get(wanted_cap)
         })
         if not cap:
@@ -397,7 +405,7 @@ class Connection(AbstractChannel):
         else:
             raise ConnectionError(
                 "Couldn't find appropriate auth mechanism "
-                "(can offer: {0}; available: {1})".format(
+                "(can offer: {}; available: {})".format(
                     b", ".join(m.mechanism
                                for m in self.authentication
                                if m.mechanism).decode(),
@@ -461,11 +469,17 @@ class Connection(AbstractChannel):
             if self._transport:
                 self._transport.close()
 
-            temp_list = [x for x in values(self.channels or {})
-                         if x is not self]
-            for ch in temp_list:
-                ch.collect()
-        except socket.error:
+            if self.channels:
+                # Copy all the channels except self since the channels
+                # dictionary changes during the collection process.
+                channels = [
+                    ch for ch in self.channels.values()
+                    if ch is not self
+                ]
+
+                for ch in channels:
+                    ch.collect()
+        except OSError:
             pass  # connection already closed on the other end
         finally:
             self._transport = self.connection = self.channels = None
@@ -475,14 +489,14 @@ class Connection(AbstractChannel):
             return self._avail_channel_ids.pop()
         except IndexError:
             raise ResourceError(
-                'No free channel ids, current={0}, channel_max={1}'.format(
+                'No free channel ids, current={}, channel_max={}'.format(
                     len(self.channels), self.channel_max), spec.Channel.Open)
 
     def _claim_channel_id(self, channel_id):
         try:
             return self._avail_channel_ids.remove(channel_id)
         except ValueError:
-            raise ConnectionError('Channel %r already open' % (channel_id,))
+            raise ConnectionError(f'Channel {channel_id!r} already open')
 
     def channel(self, channel_id=None, callback=None):
         """Create new channel.
@@ -586,7 +600,7 @@ class Connection(AbstractChannel):
                 (reply_code, reply_text, method_sig[0], method_sig[1]),
                 wait=spec.Connection.CloseOk,
             )
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             # close connection
             self.collect()
             raise
@@ -746,9 +760,10 @@ class Connection(AbstractChannel):
 
         # if we've missed two intervals' heartbeats, fail; this gives the
         # server enough time to send heartbeats a little late
-        if (self.last_heartbeat_received and
-                self.last_heartbeat_received + 2 *
-                self.heartbeat < monotonic()):
+        two_heartbeats = 2 * self.heartbeat
+        two_heartbeats_interval = self.last_heartbeat_received + two_heartbeats
+        heartbeats_missed = two_heartbeats_interval < monotonic()
+        if self.last_heartbeat_received and heartbeats_missed:
             raise ConnectionForced('Too many heartbeats missed')
 
     @property
