@@ -1,20 +1,18 @@
 """Transport implementation."""
 # Copyright (C) 2009 Barry Pederson <bp@barryp.org>
-from __future__ import absolute_import, unicode_literals
 
-import os
 import errno
+import os
 import re
 import socket
 import ssl
-from ssl import SSLError
 from contextlib import contextmanager
+from ssl import SSLError
+from struct import pack, unpack
 
 from .exceptions import UnexpectedFrame
-from .five import items
-from .platform import KNOWN_TCP_OPTS, SOL_TCP, pack, unpack
-from .utils import get_errno, set_cloexec
-
+from .platform import KNOWN_TCP_OPTS, SOL_TCP
+from .utils import set_cloexec
 
 _UNAVAIL = {errno.EAGAIN, errno.EINTR, errno.ENOENT, errno.EWOULDBLOCK}
 
@@ -25,7 +23,7 @@ EMPTY_BUFFER = bytes()
 SIGNED_INT_MAX = 0x7FFFFFFF
 
 # Yes, Advanced Message Queuing Protocol Protocol is redundant
-AMQP_PROTOCOL_HEADER = 'AMQP\x00\x00\x09\x01'.encode('latin_1')
+AMQP_PROTOCOL_HEADER = b'AMQP\x00\x00\x09\x01'
 
 # Match things like: [fe80::1]:5432, from RFC 2732
 IPV6_LITERAL = re.compile(r'\[([\.0-9a-f:]+)\](?::(\d+))?')
@@ -54,7 +52,7 @@ def to_host_port(host, default=AMQP_PORT):
     return host, port
 
 
-class _AbstractTransport(object):
+class _AbstractTransport:
     """Common superclass for TCP and SSL transports."""
 
     def __init__(self, host, connect_timeout=None,
@@ -70,6 +68,14 @@ class _AbstractTransport(object):
         self.write_timeout = write_timeout
         self.socket_settings = socket_settings
 
+    def __repr__(self):
+        if self.sock:
+            src = f'{self.sock.getsockname()[0]}:{self.sock.getsockname()[1]}'
+            dst = f'{self.sock.getpeername()[0]}:{self.sock.getpeername()[1]}'
+            return f'<{type(self).__name__}: {src} -> {dst} at {id(self):#x}>'
+        else:
+            return f'<{type(self).__name__}: (disconnected) at {id(self):#x}>'
+
     def connect(self):
         try:
             # are we already connected?
@@ -83,7 +89,7 @@ class _AbstractTransport(object):
             # EINTR, EAGAIN, EWOULDBLOCK would signal that the banner
             # has _not_ been sent
             self.connected = True
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             # if not fully connected, close socket, and reraise error
             if self.sock and not self.connected:
                 self.sock.close()
@@ -109,8 +115,8 @@ class _AbstractTransport(object):
                     # Non-blocking SSL sockets can throw SSLError
                     raise socket.timeout()
                 raise
-            except socket.error as exc:
-                if get_errno(exc) == errno.EWOULDBLOCK:
+            except OSError as exc:
+                if exc.errno == errno.EWOULDBLOCK:
                     raise socket.timeout()
                 raise
             finally:
@@ -160,7 +166,7 @@ class _AbstractTransport(object):
                         pass
                     self.sock.settimeout(timeout)
                     self.sock.connect(sa)
-                except socket.error as ex:
+                except OSError as ex:
                     e = ex
                     if self.sock is not None:
                         self.sock.close()
@@ -216,7 +222,7 @@ class _AbstractTransport(object):
         tcp_opts = self._get_tcp_socket_defaults(self.sock)
         if socket_settings:
             tcp_opts.update(socket_settings)
-        for opt, val in items(tcp_opts):
+        for opt, val in tcp_opts.items():
             self.sock.setsockopt(SOL_TCP, opt, val)
 
     def _read(self, n, initial=False):
@@ -247,6 +253,15 @@ class _AbstractTransport(object):
         self.connected = False
 
     def read_frame(self, unpack=unpack):
+        """Parse AMQP frame.
+
+        Frame has following format:
+        0      1         3         7                   size+7      size+8
+        +------+---------+---------+   +-------------+   +-----------+
+        | type | channel |  size   |   |   payload   |   | frame-end |
+        +------+---------+---------+   +-------------+   +-----------+
+         octet    short     long        'size' octets        octet
+        """
         read = self._read
         read_frame_buffer = EMPTY_BUFFER
         try:
@@ -260,7 +275,7 @@ class _AbstractTransport(object):
 
                 try:
                     part2 = read(size - SIGNED_INT_MAX)
-                except (socket.timeout, socket.error, SSLError):
+                except (socket.timeout, OSError, SSLError):
                     # In case this read times out, we need to make sure to not
                     # lose part1 when we retry the read
                     read_frame_buffer += part1
@@ -270,14 +285,14 @@ class _AbstractTransport(object):
             else:
                 payload = read(size)
             read_frame_buffer += payload
-            ch = ord(read(1))
+            frame_end = ord(read(1))
         except socket.timeout:
             self._read_buffer = read_frame_buffer + self._read_buffer
             raise
-        except (OSError, IOError, SSLError, socket.error) as exc:
+        except (OSError, SSLError) as exc:
             if (
                 isinstance(exc, socket.error) and os.name == 'nt'
-                and get_errno(exc) == errno.EWOULDBLOCK  # noqa
+                and exc.errno == errno.EWOULDBLOCK  # noqa
             ):
                 # On windows we can get a read timeout with a winsock error
                 # code instead of a proper socket.timeout() error, see
@@ -291,22 +306,23 @@ class _AbstractTransport(object):
                 self._read_buffer = read_frame_buffer + self._read_buffer
                 raise socket.timeout()
 
-            if get_errno(exc) not in _UNAVAIL:
+            if exc.errno not in _UNAVAIL:
                 self.connected = False
             raise
-        if ch == 206:  # '\xce'
+        # frame-end octet must contain '\xce' value
+        if frame_end == 206:
             return frame_type, channel, payload
         else:
             raise UnexpectedFrame(
-                'Received {0:#04x} while expecting 0xce'.format(ch))
+                f'Received frame_end {frame_end:#04x} while expecting 0xce')
 
     def write(self, s):
         try:
             self._write(s)
         except socket.timeout:
             raise
-        except (OSError, IOError, socket.error) as exc:
-            if get_errno(exc) not in _UNAVAIL:
+        except OSError as exc:
+            if exc.errno not in _UNAVAIL:
                 self.connected = False
             raise
 
@@ -317,7 +333,7 @@ class SSLTransport(_AbstractTransport):
     def __init__(self, host, connect_timeout=None, ssl=None, **kwargs):
         self.sslopts = ssl if isinstance(ssl, dict) else {}
         self._read_buffer = EMPTY_BUFFER
-        super(SSLTransport, self).__init__(
+        super().__init__(
             host, connect_timeout=connect_timeout, **kwargs)
 
     def _setup_transport(self):
@@ -340,49 +356,35 @@ class SSLTransport(_AbstractTransport):
                          server_side=False, cert_reqs=ssl.CERT_NONE,
                          ca_certs=None, do_handshake_on_connect=False,
                          suppress_ragged_eofs=True, server_hostname=None,
-                         ciphers=None, ssl_version=None):
+                         ciphers=None, ssl_version=ssl.PROTOCOL_TLS):
         """Socket wrap with SNI headers.
 
-        Default `ssl.wrap_socket` method augmented with support for
+        stdlib `ssl.SSLContext.wrap_socket` method augmented with support for
         setting the server_hostname field required for SNI hostname header
         """
-        # Setup the right SSL version; default to optimal versions across
-        # ssl implementations
-        if ssl_version is None:
-            # older versions of python 2.7 and python 2.6 do not have the
-            # ssl.PROTOCOL_TLS defined the equivalent is ssl.PROTOCOL_SSLv23
-            # we default to PROTOCOL_TLS and fallback to PROTOCOL_SSLv23
-            # TODO: Drop this once we drop Python 2.7 support
-            if hasattr(ssl, 'PROTOCOL_TLS'):
-                ssl_version = ssl.PROTOCOL_TLS
-            else:
-                ssl_version = ssl.PROTOCOL_SSLv23
-
         opts = {
             'sock': sock,
-            'keyfile': keyfile,
-            'certfile': certfile,
             'server_side': server_side,
-            'cert_reqs': cert_reqs,
-            'ca_certs': ca_certs,
             'do_handshake_on_connect': do_handshake_on_connect,
             'suppress_ragged_eofs': suppress_ragged_eofs,
-            'ciphers': ciphers,
-            'ssl_version': ssl_version
+            'server_hostname': server_hostname,
         }
 
-        sock = ssl.wrap_socket(**opts)
+        context = ssl.SSLContext(ssl_version)
+        if certfile is not None:
+            context.load_cert_chain(certfile, keyfile)
+        if ca_certs is not None:
+            context.load_verify_locations(ca_certs)
+        if ciphers:
+            context.set_ciphers(ciphers)
+        if cert_reqs != ssl.CERT_NONE:
+            context.check_hostname = True
         # Set SNI headers if supported
         if (server_hostname is not None) and (
                 hasattr(ssl, 'HAS_SNI') and ssl.HAS_SNI) and (
                 hasattr(ssl, 'SSLContext')):
-            context = ssl.SSLContext(opts['ssl_version'])
             context.verify_mode = cert_reqs
-            if cert_reqs != ssl.CERT_NONE:
-                context.check_hostname = True
-            if (certfile is not None) and (keyfile is not None):
-                context.load_cert_chain(certfile, keyfile)
-            sock = context.wrap_socket(sock, server_hostname=server_hostname)
+        sock = context.wrap_socket(**opts)
         return sock
 
     def _shutdown_transport(self):
@@ -401,11 +403,7 @@ class SSLTransport(_AbstractTransport):
             while len(rbuf) < n:
                 try:
                     s = recv(n - len(rbuf))  # see note above
-                except socket.error as exc:
-                    # ssl.sock.read may cause a SSLerror without errno
-                    # http://bugs.python.org/issue10272
-                    if isinstance(exc, SSLError) and 'timed out' in str(exc):
-                        raise socket.timeout()
+                except OSError as exc:
                     # ssl.sock.read may cause ENOENT if the
                     # operation couldn't be performed (Issue celery#1414).
                     if exc.errno in _errnos:
@@ -414,7 +412,7 @@ class SSLTransport(_AbstractTransport):
                         continue
                     raise
                 if not s:
-                    raise IOError('Server unexpectedly closed connection')
+                    raise OSError('Server unexpectedly closed connection')
                 rbuf += s
         except:  # noqa
             self._read_buffer = rbuf
@@ -435,7 +433,7 @@ class SSLTransport(_AbstractTransport):
                 # None.
                 n = 0
             if not n:
-                raise IOError('Socket closed')
+                raise OSError('Socket closed')
             s = s[n:]
 
 
@@ -457,14 +455,14 @@ class TCPTransport(_AbstractTransport):
             while len(rbuf) < n:
                 try:
                     s = recv(n - len(rbuf))
-                except socket.error as exc:
+                except OSError as exc:
                     if exc.errno in _errnos:
                         if initial and self.raise_on_initial_eintr:
                             raise socket.timeout()
                         continue
                     raise
                 if not s:
-                    raise IOError('Server unexpectedly closed connection')
+                    raise OSError('Server unexpectedly closed connection')
                 rbuf += s
         except:  # noqa
             self._read_buffer = rbuf
